#include "main.h"

#define		tCSCS		10

/****************************************************************************************************************************************
*	Function name:  Init_ADCC_Regs
*	Description	:	This function configure different ADCC registers including:
*					ADCC TIming control registers: ADC_CLK = SCLK/2 = 50MHz. TCSCK = 1, NCK = 8, TCKCS = 0 and TCSCS as configurable in above macro
*					ADCC Control register: The guidelines for configuring this register is given in Hardware refernce manual
*					Event Control and Event Time registers: 24 events are enabled. Event0-11 are assigned to ADCC Timer0  and EVT12-23 are assigned to Timer1
*					DMA registers: 
*	Arguments	:	None
*   Return value:   None.
****************************************************************************************************************************************/

void Init_ADCC_Regs(void)
{
	*pREG_ADCC0_CTL = 0x0000;						//	disable ADCC first
	while(*pREG_ADCC0_CFG & BITM_ADCC_CFG_PND);		//	wait till ADCC operation is pending
	
    *pREG_ADCC0_TCA0 = (((1 << BITP_ADCC_TCA0_CKDIV ) & BITM_ADCC_TCA0_CKDIV ) |
						((8 << BITP_ADCC_TCA0_NCK   ) & BITM_ADCC_TCA0_NCK   ) );
    *pREG_ADCC0_TCB0 = (((1 << BITP_ADCC_TCB0_TCSCK ) & BITM_ADCC_TCB0_TCSCK ) | 
						((0 << BITP_ADCC_TCB0_TCKCS ) & BITM_ADCC_TCB0_TCKCS ) |
						((tCSCS << BITP_ADCC_TCB0_TCSCS ) & BITM_ADCC_TCB0_TCSCS ) );
 
    *pREG_ADCC0_TCA1 = (((1 << BITP_ADCC_TCA1_CKDIV ) & BITM_ADCC_TCA1_CKDIV ) |
						((8 << BITP_ADCC_TCA1_NCK   ) & BITM_ADCC_TCA1_NCK   ) );
    *pREG_ADCC0_TCB1 = (((1 << BITP_ADCC_TCB0_TCSCK ) & BITM_ADCC_TCB0_TCSCK ) | 
						((0 << BITP_ADCC_TCB0_TCKCS ) & BITM_ADCC_TCB0_TCKCS ) |
						((tCSCS << BITP_ADCC_TCB0_TCSCS ) & BITM_ADCC_TCB0_TCSCS ) );
	
	// LSBF0, LSBF1, DSWP0, DSWP1, TIDLE0, TIDLE1, CSPOL0, CSPOL1, CKPOL0, CKPOL1 = 0
	// Timer0 enabled by default, 	TRGSEL1 = TRG1 (ID:25);   TRGPOL1 = +ve edge TRG; 	TRGOE1 depends on Init_Slave_TMR bits
	// Timer1 enable: TMR1EN=0, 	TRGSEL1 = TRG2 (ID:26);   TRGPOL1 = +ve edge TRG; 	TRGOE1 depends on Init_Slave_TMR bits
	// CSIZE = 0, DSIZE = 1, CFG = 0;
	*pREG_ADCC0_CTL	= ( ((1 << BITP_ADCC_CTL_TRGSEL0) & BITM_ADCC_CTL_TRGSEL0)	|
					   	((1 << BITP_ADCC_CTL_TRGOE0 ) & BITM_ADCC_CTL_TRGOE0 ) 	| 
					    ((1 << BITP_ADCC_CTL_TMR1EN ) & BITM_ADCC_CTL_TMR1EN )	|
						((2 << BITP_ADCC_CTL_TRGSEL1) & BITM_ADCC_CTL_TRGSEL1)	|
						((1 << BITP_ADCC_CTL_TRGOE1 ) & BITM_ADCC_CTL_TRGOE1 ) 	| 
						((1 << BITP_ADCC_CTL_DSIZE  ) & BITM_ADCC_CTL_DSIZE  )	|
						((0 << BITP_ADCC_CTL_CSIZE  ) & BITM_ADCC_CTL_CSIZE  )	|
						((1 << BITP_ADCC_CTL_DMAEN  ) & BITM_ADCC_CTL_DMAEN  )	);

	int EVT_ID;
	volatile unsigned int *pADCC_REG_Ptr;
//	Timer0	
	for(EVT_ID=0; EVT_ID < 12; EVT_ID++)
	{
		pADCC_REG_Ptr 	= (unsigned int *)  REG_ADCC0_EVT00 + (2 * EVT_ID);
		*pADCC_REG_Ptr 	= ( EVT_ID * (2*(tCSCS + 8 + 1)) )-20;					//	where 2 = SCLK/ADC_CLK
    	
		pADCC_REG_Ptr 	= (unsigned int *)  REG_ADCC0_EVCTL00 + (2 * EVT_ID);
		*pADCC_REG_Ptr 	= ( ((0 		<< BITP_ADCC_EVCTL_TMRSEL) & BITM_ADCC_EVCTL_TMRSEL ) |
							((ADC_SEL	<< BITP_ADCC_EVCTL_ADCSEL) & BITM_ADCC_EVCTL_ADCSEL ) |
							((ADC_CHNL	<< BITP_ADCC_EVCTL_CHN_ID) & BITM_ADCC_EVCTL_CHN_ID ) |
							(((2*EVT_ID)<< BITP_ADCC_EVCTL_EVTOFS) & BITM_ADCC_EVCTL_EVTOFS ) | 0xD);
	}	
	*pREG_ADCC0_EVT00 = 0x00;
	
//	Timer1	
	for(EVT_ID=12; EVT_ID < 24; EVT_ID++)
	{
		pADCC_REG_Ptr 	= (unsigned int *)  REG_ADCC0_EVT00 + (2 * EVT_ID);
		*pADCC_REG_Ptr 	= ( (EVT_ID-12) * (2*(tCSCS + 8 + 1)) )-20;					//	where 2 = SCLK/ADC_CLK
    	
		pADCC_REG_Ptr 	= (unsigned int *)  REG_ADCC0_EVCTL00 + (2 * EVT_ID);
		*pADCC_REG_Ptr 	= ( ((1 		<< BITP_ADCC_EVCTL_TMRSEL) & BITM_ADCC_EVCTL_TMRSEL ) |
							((ADC_SEL  	<< BITP_ADCC_EVCTL_ADCSEL) & BITM_ADCC_EVCTL_ADCSEL ) |
							((ADC_CHNL 	<< BITP_ADCC_EVCTL_CHN_ID) & BITM_ADCC_EVCTL_CHN_ID ) |
							(((2*EVT_ID)<< BITP_ADCC_EVCTL_EVTOFS) & BITM_ADCC_EVCTL_EVTOFS ) | 0xD);
	}	
	*pREG_ADCC0_EVT12 = 0x00;
	
	*pREG_ADCC0_EVTEN_SET = 0xFFFFFF; 	
	
	*pREG_ADCC0_BPTR0  = (unsigned int) ADC_Data;
	*pREG_ADCC0_FRINC0 = 24 *2 ;
	*pREG_ADCC0_CBSIZ0 = BUFF_SIZE ;
	*pREG_ADCC0_BWMON0 = 0;
		
	*pREG_ADCC0_BPTR1  = (unsigned int) ADC_Data;
	*pREG_ADCC0_FRINC1 = 24 * 2 ;
	*pREG_ADCC0_CBSIZ1 = BUFF_SIZE ;
	*pREG_ADCC0_BWMON1 = 0;
}


/****************************************************************************************************************************************
*	Function name:  Enable_ADCC
*	Description	:	This function simply enables ADCC module. This function is called after configuring all other ADCC registers.
*	Arguments	:	None
*   Return value:   None.
****************************************************************************************************************************************/

void Enable_ADCC(void)
{
	*pREG_ADCC0_CTL |= BITM_ADCC_CTL_TRGIE0 | BITM_ADCC_CTL_TRGIE1 ;
	*pREG_ADCC0_CTL |= BITM_ADCC_CTL_EN;
}


/****************************************************************************************************************************************
*	Function name:  Init_ADCC_Timer_trigger
*	Description	:	The trigger inputs for ADCC Timers are generated internally using GP Timer. The trigger output & delay feature of the
*					GP TImers is especially useful to create the timing specified in EE_Note
*	Arguments	:	None
*   Return value:   None.
****************************************************************************************************************************************/

void Init_ADCC_Timer_trigger(void)
{
	*pREG_TIMER0_RUN_CLR = BITM_TIMER_RUN_SET_TMR04 | BITM_TIMER_RUN_SET_TMR05;

   	*pREG_TRU0_GCTL  = ((1 << BITP_TRU_GCTL_EN) & BITM_TRU_GCTL_EN);
	*pREG_TRU0_SSR25 = ((TRGM_TIMER0_TMR6 << BITP_TRU_SSR_SSR) & BITM_TRU_SSR_SSR);
	*pREG_TRU0_SSR26 = ((TRGM_TIMER0_TMR7 << BITP_TRU_SSR_SSR) & BITM_TRU_SSR_SSR);
	
	unsigned int TMR_PERIOD = 24 * (tCSCS + 8 + 1) * 2;
//Init Timer Registers
	*pREG_TIMER0_TMR6_CFG =(ENUM_TIMER_TMR_CFG_IRQMODE1	|
							ENUM_TIMER_TMR_CFG_POS_EDGE |
							ENUM_TIMER_TMR_CFG_PADOUT_DIS | 
							ENUM_TIMER_TMR_CFG_PWMCONT_MODE);
	*pREG_TIMER0_TMR6_DLY = TMR_PERIOD/2;
	*pREG_TIMER0_TMR6_WID = 10;
	*pREG_TIMER0_TMR6_PER = TMR_PERIOD;

	*pREG_TIMER0_TMR7_CFG =(ENUM_TIMER_TMR_CFG_IRQMODE1	|
							ENUM_TIMER_TMR_CFG_POS_EDGE |
							ENUM_TIMER_TMR_CFG_PADOUT_DIS |
							ENUM_TIMER_TMR_CFG_PWMCONT_MODE	);
	*pREG_TIMER0_TMR7_DLY = TMR_PERIOD-20 ;
	*pREG_TIMER0_TMR7_WID = 10 ;
	*pREG_TIMER0_TMR7_PER = TMR_PERIOD;
	
	*pREG_TIMER0_TRG_MSK 	 &= ~(BITM_TIMER_TRG_MSK_TMR06 | BITM_TIMER_TRG_MSK_TMR07) ;
	*pREG_TIMER0_STOP_CFG_SET =(BITM_TIMER_STOP_CFG_TMR06 | BITM_TIMER_STOP_CFG_TMR07);
	*pREG_TIMER0_RUN_SET      =  (BITM_TIMER_RUN_SET_TMR06   | BITM_TIMER_RUN_SET_TMR07) ;
	*pREG_TIMER0_TMR4_DLY 	 -= 20;
}


/****************************************************************************************************************************************
*	Function name:  Init_AFE 
*	Description	:	This function initialises the Analog subsystem of the Analog die. This is also used to configure internal reference 
*					or external reference. This function should be called once at the startup.
*	Arguments	:	None
*   Return value:   None.
****************************************************************************************************************************************/

void Init_AFE(void)
{
    *pREG_ADCC0_TCA0 = (((1 << BITP_ADCC_TCA0_CKDIV ) & BITM_ADCC_TCA0_CKDIV ) |
						((8 << BITP_ADCC_TCA0_NCK   ) & BITM_ADCC_TCA0_NCK   ) );
    *pREG_ADCC0_TCB0 = (((1 << BITP_ADCC_TCB0_TCSCK ) & BITM_ADCC_TCB0_TCSCK ) | 
						((0 << BITP_ADCC_TCB0_TCKCS ) & BITM_ADCC_TCB0_TCKCS ) |
						((11 << BITP_ADCC_TCB0_TCSCS ) & BITM_ADCC_TCB0_TCSCS ) );
 
    *pREG_ADCC0_TCA1 = (((1 << BITP_ADCC_TCA1_CKDIV ) & BITM_ADCC_TCA1_CKDIV ) |
						((8 << BITP_ADCC_TCA1_NCK   ) & BITM_ADCC_TCA1_NCK   ) );
    *pREG_ADCC0_TCB1 = (((1 << BITP_ADCC_TCB0_TCSCK ) & BITM_ADCC_TCB0_TCSCK ) | 
						((0 << BITP_ADCC_TCB0_TCKCS ) & BITM_ADCC_TCB0_TCKCS ) |
						((11 << BITP_ADCC_TCB0_TCSCS ) & BITM_ADCC_TCB0_TCSCS ) );

    //	Configure ADCC_CFG register
	*pREG_ADCC0_CFG = (	((EXT_REF << BITP_ADCC_CFG_REFSEL  ) & BITM_ADCC_CFG_REFSEL ) );
  
    //	Enable ADCC with 
	// LSBF0, LSBF1, DSWP0, DSWP1, TIDLE0, TIDLE1, CSPOL0, CSPOL1, CKPOL0, CKPOL1 = 0
	// TRGIE0, TRIGIE1 = 0. So, TRGSEL0, TRGSEL1, TRGPOL0, TRGPOL1, TRGOEN0, TRGOEN1 will xx 	   
	// TMR1EN, DMAEN = 0. CSIZE = 0, DSIZE = 1, CFG = 1;	   
	*pREG_ADCC0_CTL	= (BITM_ADCC_CTL_EN | BITM_ADCC_CTL_ADCFG | BITM_ADCC_CTL_DSIZE );
	
    // wait till ADC configuration is pending	
	while(!(*pREG_ADCC0_CFG & BITM_ADCC_CFG_PND));	//	wait for PND bit to set and then wait till it clears
	while(*pREG_ADCC0_CFG & BITM_ADCC_CFG_PND);		//	wait till ADCC operation is pending
	
    //	wait for 'Fuse-load Time' required by AFC
	int wait_cnt;
	for(wait_cnt=0; wait_cnt< 0x3000000; wait_cnt++);
	
    //	Disable ADCC
	*pREG_ADCC0_CTL = 0x0000;				//	disable ADCC first
	while(*pREG_ADCC0_CFG & BITM_ADCC_CFG_PND);		//	wait till ADCC operation is pending
}
/**********************************************************************************************************************************/

